'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ResponseBuilder = require('./response-builder');
var requestUtils = require('./request-utils');
var FetchMock = {};

var normalizeRequest = function normalizeRequest(url, options, Request) {
	if (Request.prototype.isPrototypeOf(url)) {
		var obj = {
			url: requestUtils.normalizeUrl(url.url),
			opts: {
				method: url.method
			},
			request: url
		};

		var headers = requestUtils.headers.toArray(url.headers);

		if (headers.length) {
			obj.opts.headers = requestUtils.headers.zip(headers);
		}
		return obj;
	} else if (typeof url === 'string' ||
	// horrible URL object duck-typing
	(typeof url === 'undefined' ? 'undefined' : (0, _typeof3.default)(url)) === 'object' && 'href' in url) {
		return {
			url: requestUtils.normalizeUrl(url),
			opts: options
		};
	} else if ((typeof url === 'undefined' ? 'undefined' : (0, _typeof3.default)(url)) === 'object') {
		throw new TypeError('fetch-mock: Unrecognised Request object. Read the Config and Installation sections of the docs');
	} else {
		throw new TypeError('fetch-mock: Invalid arguments passed to fetch');
	}
};

var resolve = function () {
	var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(response, url, opts) {
		return _regenerator2.default.wrap(function _callee$(_context) {
			while (1) {
				switch (_context.prev = _context.next) {
					case 0:
						if (!(typeof response === 'function' || typeof response.then === 'function')) {
							_context.next = 10;
							break;
						}

						if (!(typeof response === 'function')) {
							_context.next = 5;
							break;
						}

						response = response(url, opts);
						_context.next = 8;
						break;

					case 5:
						_context.next = 7;
						return response;

					case 7:
						response = _context.sent;

					case 8:
						_context.next = 0;
						break;

					case 10:
						return _context.abrupt('return', response);

					case 11:
					case 'end':
						return _context.stop();
				}
			}
		}, _callee, undefined);
	}));

	return function resolve(_x, _x2, _x3) {
		return _ref.apply(this, arguments);
	};
}();

FetchMock.fetchHandler = function (url, opts, request) {
	var _this = this;

	var _normalizeRequest = normalizeRequest(url, opts, this.config.Request);

	url = _normalizeRequest.url;
	opts = _normalizeRequest.opts;
	request = _normalizeRequest.request;


	var route = this.executeRouter(url, opts, request);

	// this is used to power the .flush() method
	var done = void 0;
	this._holdingPromises.push(new this.config.Promise(function (res) {
		return done = res;
	}));

	// wrapped in this promise to make sure we respect custom Promise
	// constructors defined by the user
	return new this.config.Promise(function (res, rej) {
		if (opts && opts.signal) {
			opts.signal.addEventListener('abort', function () {
				rej(new Error('URL \'' + url + '\' aborted.'));
				done();
			});
		}

		_this.generateResponse(route, url, opts).then(res, rej).then(done, done);
	});
};

FetchMock.fetchHandler.isMock = true;

FetchMock.executeRouter = function (url, options, request) {
	if (this.config.fallbackToNetwork === 'always') {
		return { response: this.getNativeFetch() };
	}

	var match = this.router(url, options, request);

	if (match) {
		return match;
	}

	if (this.config.warnOnFallback) {
		console.warn('Unmatched ' + (options && options.method || 'GET') + ' to ' + url); // eslint-disable-line
	}

	this.push({ url: url, options: options, request: request, unmatched: true });

	if (this.fallbackResponse) {
		return { response: this.fallbackResponse };
	}

	if (!this.config.fallbackToNetwork) {
		throw new Error('fetch-mock: No fallback response defined for ' + (options && options.method || 'GET') + ' to ' + url);
	}

	return { response: this.getNativeFetch() };
};

FetchMock.generateResponse = function () {
	var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(route, url, opts) {
		var response;
		return _regenerator2.default.wrap(function _callee2$(_context2) {
			while (1) {
				switch (_context2.prev = _context2.next) {
					case 0:
						_context2.next = 2;
						return resolve(route.response, url, opts);

					case 2:
						response = _context2.sent;

						if (!(response.throws && typeof response !== 'function')) {
							_context2.next = 5;
							break;
						}

						throw response.throws;

					case 5:
						if (!this.config.Response.prototype.isPrototypeOf(response)) {
							_context2.next = 7;
							break;
						}

						return _context2.abrupt('return', response);

					case 7:
						return _context2.abrupt('return', new ResponseBuilder({
							url: url,
							shorthandResponse: response,
							fetchMock: this,
							route: route
						}).exec());

					case 8:
					case 'end':
						return _context2.stop();
				}
			}
		}, _callee2, this);
	}));

	return function (_x4, _x5, _x6) {
		return _ref2.apply(this, arguments);
	};
}();

FetchMock.router = function (url, options, request) {
	var route = this.routes.find(function (route) {
		return route.matcher(url, options, request);
	});

	if (route) {
		this.push({
			url: url,
			options: options,
			request: request,
			identifier: route.identifier
		});
		return route;
	}
};

FetchMock.getNativeFetch = function () {
	var func = this.realFetch || this.isSandbox && this.config.fetch;
	if (!func) {
		throw new Error('fetch-mock: Falling back to network only available on gloabl fetch-mock, or by setting config.fetch on sandboxed fetch-mock');
	}
	return func;
};

FetchMock.push = function (_ref3) {
	var url = _ref3.url,
	    options = _ref3.options,
	    request = _ref3.request,
	    unmatched = _ref3.unmatched,
	    identifier = _ref3.identifier;

	var args = [url, options];
	args.request = request;
	args.identifier = identifier;
	args.unmatched = unmatched;
	this._calls.push(args);
};

module.exports = FetchMock;